import http from 'k6/http'
import { check } from 'k6'

const allChecksNeedToPassTreshold = { checks: [{ threshold: 'rate==1', abortOnFail: true }] }

export const options = {
  vus: 1,
  iterations: 1,
  thresholds: allChecksNeedToPassTreshold,
}

function forQuery(query, checkFunction) {
  const response = http.post(`${__ENV.BASE_URL}`, JSON.stringify({ query }), {
    headers: { 'Content-Type': 'application/json' },
  })
  checkFunction(response)
}

export default () => {
  forQuery(`{ events(limit:1) { data { id } } }`, (response) => {
    check(response, {
      'is status 200': (r) => r.status === 200,
    })

    check(JSON.parse(response.body), {
      'contains events': (r) => Array.isArray(r.data.events.data) && r.data.events.data.length > 0,
    })
  })
}
